/* ============================================================================
 Name        : LinkedList.c
 Author      : Alex Nelson
 Version     :
 Copyright   :
 Description : Starter list file for In-Class Assignment
 ============================================================================ */
#include "LinkedList.h"

//Create a new node using malloc and return the pointer to it
NODE * CreateNode(enum DataType type, void* newdata) {
	NODE * newnode;
	LIST * myList;
	int data;
	newnode = (NODE *) malloc(sizeof(NODE));

	if (newnode == NULL) {
		printf("No Memory left !!\n");
		return NULL;
	}
	switch (type) {
	case INTEGER_ENUM:
		//set data of node to the integer passed to the function
		data = *(int*)newdata;
		newnode->data.data_i = data;
		//initialize next pointer to NULL
		break;
	case LIST_ENUM:
		myList = (LIST*)newdata;
		newnode->data.list = myList;

		break;
	}
	newnode->type=type;
	newnode->prev = NULL;
	newnode->prev = NULL;
	return newnode;
}

//Create a new list using malloc and return the pointer to it
LIST * CreateList() {
	//malloc a new list
	LIST * myList = (LIST *) malloc(sizeof(LIST));
	//Set initial head pointer to NULL
	myList->head = NULL;
	myList->tail = NULL;
	//Initialize size to 0
	myList->size = 0;
	return myList;
}

//Delete a node at given index and free it
int DeleteIndex(LIST * myList, int index) {
	//Create pointers to nodes, prev and curr
	NODE* prevNode, *currNode;
	//set curr node to head
	currNode = myList->head;
	if (currNode == NULL) { //if list is empty
		printf("List must not be empty!\n");
		return 1;
	} else if (index > myList->size - 1) { //index is greater than list size
		printf("List is not that big\n");
		return 1;
	} else if (myList->size == 1) { //if list is size 1, set head pointer to null
		myList->head = NULL;
		myList->tail = NULL;
		free(currNode);
		myList->size--;
		return 0;
	} else if (index == 0) { //if index is 0, change head pointer to next
		myList->head = currNode->next;
		currNode->next->prev = NULL;
		free(currNode);
		myList->size--;
		return 0;
	} else if (index == myList->size - 1) {
		currNode = myList->tail;
		currNode->prev->next = NULL;
		myList->tail = currNode->prev;
		free(currNode);
		myList->size--;
		return 0;
	} else if (index > 0) { //as long as index is positive, iterate through and change pointer
		int i = 0;
		prevNode = currNode;
		for (i = 0; i < index - 1; i++) {
			prevNode = prevNode->next;
		}
		currNode = prevNode->next;
		prevNode->next = currNode->next;
		currNode->next->prev = prevNode;
		free(currNode);
		myList->size--;
		return 0;
	} else {
		printf("Must be a positive number\n");
		return 1;
	}
}
//Wrapper function to create a new node and insert into the list
void InsertInt(LIST * myList, int pos, int val) {
	//create new node with value
	NODE * myNode = CreateNode(INTEGER_ENUM, &val);
	//Insert node into that position
	InsertNode(myList, myNode, pos);
}

void InsertList(LIST * wrapperList, int pos, LIST * myList){
	NODE* myNode = CreateNode(LIST_ENUM, myList);
	InsertNode(wrapperList,myNode,pos);
}
//Insert a new node into the list at a given position
void InsertNode(LIST * myList, NODE * newdata, int pos) {
	//create pointers to curr and prev nodes
	//set currnode to point at head
	NODE* currNode = myList->head, *prevNode;
	if (myList->size == 0 && pos == 0) { //If list size is 0, and pos = 0 set head pointer
		myList->head = newdata;  //head pointer = new node
		newdata->next = NULL; //new node next is null
		newdata->prev = NULL;
		myList->tail = newdata;
		myList->size = 1; //size is now 1
	} else if (pos > myList->size || pos < 0) {
		printf("Cannot be inserted outside list bounds\n");
	} else if (pos == 0) {  //if pos = 0, set head node
		myList->head = newdata;  //head node points to new data
		newdata->next = currNode; //new data next points to head node
		newdata->prev = NULL;
		myList->size = myList->size + 1; //increase size of list by 1
	} else if (pos == myList->size) {
		currNode = myList->tail;
		myList->tail = newdata;
		newdata->prev = currNode;
		newdata->next = NULL;
		currNode->next = newdata;
		myList->size = myList->size + 1;
	} else if (pos < myList->size) {  //as long as in the range to be added
		int i;
		prevNode = currNode;  //set prev node to head node
		for (i = 0; i < pos - 1; i++) {
			prevNode = prevNode->next; //iterate to point at node before insert point
		}
		currNode = prevNode->next; //set curr node to point at node after insert point
		newdata->next = currNode;  //set new data next to point to currNode
		prevNode->next = newdata;  //set prev node next to point to next data
		newdata->prev = prevNode;
		currNode->prev = newdata;
		myList->size = myList->size + 1;  //increase list size by 1
	}
}

//Print the linked list starting with the node pointed to by the head
void PrintList(LIST * myList) {
	//create pointer to current
	NODE * curr;
	if (myList->head == NULL) {  //if list is empty
		printf("Empty List !!\n");  //print empty list
	} else {
		//curr points to the first node in the list
		curr = myList->head;
		printf("List of nodes: ");
		while (curr != NULL) {  //iterate until end of list
			if(curr->type==INTEGER_ENUM){
				printf("%d, ", curr->data.data_i);  //print current node
				curr = curr->next; //Move to the next node
			}
			else{
				PrintList(curr->data.list);
				curr = curr->next;
			}
		}
		printf("\n");  //print new line
	}
}
void PrintBackwards(LIST * myList) {
	//create pointer to current
	NODE * curr;
	if (myList->head == NULL) {  //if list is empty
		printf("Empty List !!\n");  //print empty list
	} else {
		//curr points to the first node in the list
		curr = myList->tail;
		printf("List of nodes backwards: ");
		while (curr != NULL) {  //iterate until end of list
			if(curr->type==INTEGER_ENUM){
				printf("%d ", curr->data.data_i);  //print current node
				curr = curr->prev; //Move to the next node
			}
			else{
				PrintList(curr->data.list);
				curr = curr->prev;
			}
		}
		printf("\n");  //print new line
	}
}

void DeleteList(LIST* myList) {
	NODE * curr = myList->head;
	NODE * prev = curr;
	while (curr != NULL) {
		curr = curr->next;
		if(prev->type==LIST_ENUM){
			DeleteList(prev->data.list);
		}
		free(prev);
		prev = curr;
	}
	free(myList);
}

